@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2012 FreeIPMI Core Team
+ * Copyright (C) 2003-2013 FreeIPMI Core Team
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -71,12 +71,16 @@ enum argp_common_option_keys
     ARGP_SDR_CACHE_FILE_KEY = 148,
     ARGP_SDR_CACHE_DIRECTORY_KEY = 150,
     ARGP_IGNORE_SDR_CACHE_KEY = 151,
+    /* time options */
+    ARGP_UTC_TO_LOCALTIME_KEY = 152,
+    ARGP_LOCALTIME_TO_UTC_KEY = 153,
+    ARGP_UTC_OFFSET_KEY = 154,
     /* hostrange options */
     ARGP_BUFFER_OUTPUT_KEY = 'B',
     ARGP_CONSOLIDATE_OUTPUT_KEY = 'C',
     ARGP_FANOUT_KEY = 'F',
     ARGP_ELIMINATE_KEY = 'E',
-    ARGP_ALWAYS_PREFIX_KEY = 152,
+    ARGP_ALWAYS_PREFIX_KEY = 155,
   };
 
 /*
@@ -190,21 +194,29 @@ enum argp_common_option_keys
   { "ignore-sdr-cache", ARGP_IGNORE_SDR_CACHE_KEY, 0, 0,                                                        \
       "Ignore all SDR cache related processing.", 25}
 
+#define ARGP_COMMON_TIME_OPTIONS                                                                                \
+  { "utc-to-localtime", ARGP_UTC_TO_LOCALTIME_KEY, 0, 0,                                                        \
+      "Assume times stored UTC, convert to localtime.", 26},                                                    \
+  { "localtime-to-utc", ARGP_LOCALTIME_TO_UTC_KEY, 0, 0,                                                        \
+      "Assume times stored in localtime, convert to UTC.", 27},                                                 \
+  { "utc-offset", ARGP_UTC_OFFSET_KEY, "SECONDS", 0,                                                            \
+      "Specify a specific UTC offset to be added to timestamps.", 28}
+
 #define ARGP_COMMON_HOSTRANGED_OPTIONS                                                                          \
   { "buffer-output", ARGP_BUFFER_OUTPUT_KEY, 0, 0,                                                              \
-      "Buffer hostranged output.", 26},                                                                         \
+      "Buffer hostranged output.", 29},                                                                         \
   { "consolidate-output", ARGP_CONSOLIDATE_OUTPUT_KEY, 0, 0,                                                    \
-      "Consolidate hostranged output.", 27},                                                                    \
+      "Consolidate hostranged output.", 30},                                                                    \
   { "fanout", ARGP_FANOUT_KEY, "NUM", 0,                                                                        \
-      "Specify multiple host fanout.", 28},                                                                     \
+      "Specify multiple host fanout.", 31},                                                                     \
   { "eliminate", ARGP_ELIMINATE_KEY, 0, 0,                                                                      \
-      "Eliminate undetected nodes.", 29},                                                                       \
+      "Eliminate undetected nodes.", 32},                                                                       \
   { "always-prefix", ARGP_ALWAYS_PREFIX_KEY, 0, 0,                                                              \
-      "Always prefix output.", 30}
+      "Always prefix output.", 33}
 
 #define ARGP_COMMON_OPTIONS_DEBUG                                                                               \
   { "debug",     ARGP_DEBUG_KEY, 0, 0,                                                                          \
-      "Turn on debugging.", 31}
+      "Turn on debugging.", 34}
 
 struct common_cmd_args
 {
@@ -251,6 +263,11 @@ struct common_cmd_args
   char *sdr_cache_directory;
   int ignore_sdr_cache;
 
+  /* time options */
+  int utc_to_localtime;
+  int localtime_to_utc;
+  int utc_offset;
+
   /* hostrange options */
   int buffer_output;
   int consolidate_output;
